function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var React = require('react');
var React__default = _interopDefault(React);

var hasOwnProperty = Object.prototype.hasOwnProperty;
function is(x, y) {
    if (x === y) {
        return x !== 0 || y !== 0 || 1 / x === 1 / y;
    } else {
        return x !== x && y !== y;
    }
}

function shallowEqual(objA, objB) {
    if (is(objA, objB)) {
        return true;
    }
    if (typeof objA !== "object" || objA === null || typeof objB !== "object" || objB === null) {
        return false;
    }
    var keysA = Object.keys(objA);
    var keysB = Object.keys(objB);
    if (keysA.length !== keysB.length) {
        return false;
    }
    for (var i = 0;i < keysA.length; i++) {
        if (!hasOwnProperty.call(objB, keysA[i]) || !is(objA[keysA[i]], objB[keysA[i]])) {
            return false;
        }
    }
    return true;
}

var SubstateContext = React__default.createContext();
var SubstateProvider = SubstateContext.Provider;
function useSubstate(selectSubstate) {
    var store = React.useContext(SubstateContext);
    var ref = React.useState(function () {
        var state = store.getState();
        return selectSubstate(state);
    });
    var substate = ref[0];
    var setSubstate = ref[1];
    var prevSubstateRef = React.useRef();
    React.useEffect(function () {
        prevSubstateRef.current = substate;
    });
    React.useEffect(function () {
        var checkForUpdates = function () {
            var nextState = store.getState();
            var nextSubstate = selectSubstate(nextState);
            if (shallowEqual(prevSubstateRef.current, nextSubstate)) {
                return;
            }
            setSubstate(nextSubstate);
        };
        checkForUpdates();
        return store.subscribe(checkForUpdates);
    }, [store,selectSubstate]);
    return [substate,store.dispatch];
}

exports.SubstateProvider = SubstateProvider;
exports.useSubstate = useSubstate;
